package com.iden.bms.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.iden.bms.service.SysRoleService;
import com.iden.common.annotation.Permission;
import com.iden.common.exception.BDException;
import com.iden.common.util.PageResult;
import com.iden.common.util.Result;
import com.iden.common.util.WebPageUtils;
import com.iden.common.vo.SysRoleVO;
import com.iden.common.vo.PageReqVO;
import com.iden.common.vo.UserLogindConvertVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @Author: lilt
 * @Date: 2021/5/26
 * @Desc:
 */
@RestController
@Api(value = "SysRoleController", tags = { "角色管理" })
@Slf4j
@RequestMapping("/bms/sysRole")
@Permission
public class SysRoleController {

    @Autowired
    private SysRoleService sysRoleService;



    @GetMapping("/listSysRole")
    @ApiOperation(value = "角色列表分页 ")
    public PageResult<List<SysRoleVO>> listSysRole(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                                  PageReqVO pageReqVo){
        try {
            UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            IPage<SysRoleVO> pageResponse = this.sysRoleService.listSysRole(loginUser ,pageReqVo);
            return PageResult.success(pageResponse.getRecords(),pageResponse.getCurrent(),pageResponse.getSize(),pageResponse.getTotal());
        }catch (BDException e) {
            log.error("角色列表查询-分页列表出现异常",e);
            return  PageResult.error(e.getMessage());
        } catch (Exception e) {
            log.error("角色管理: 角色列表查询出现异常",e);
            return PageResult.error( "获取列表失败");
        }
    }

    @PostMapping("/addSysRole")
    @ApiOperation(value = "新增角色")
    public Result<Object> addSysRole(HttpServletRequest request,@RequestHeader("token") String token,
                                   @RequestBody SysRoleVO vo){
        try {

            UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            int flag = this.sysRoleService.createSysRole(vo,loginUser);
            if (flag == 1) {
                return  Result.error("名称已存在！");
            }  if (flag == 2) {
                return  Result.error("编码已存在！");
            }  else if (flag == 0) {
                return  Result.success("新增成功！");
            } else {
                return  Result.error("新增失败！");
            }

        }catch (BDException e) {
            log.error("新增角色-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("角色管理: 新增角色出现异常",e);
            return Result.error("新增失败！");
        }
    }

    @PostMapping("/updateSysRole")
    @ApiOperation(value = "修改角色")
    public Result<Object> updateSysRole(@RequestHeader("token") String token,
                                 @RequestBody SysRoleVO vo){
        try {
            this.sysRoleService.updateSysRole(vo);
            return  Result.success("修改成功！");
        }catch (BDException e) {
            log.error("修改角色-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("角色管理: 修改角色出现异常",e);
            return Result.error("修改失败！");
        }
    }

    @GetMapping("/getSysRoleInfo/{id}")
    @ApiOperation(value = "角色详情")
    public Result<SysRoleVO> getSysRoleInfo(HttpServletRequest request, @RequestHeader("token") String token, @PathVariable("id") Long id){
        SysRoleVO orderInfo = this.sysRoleService.getSysRoleById(id);
        return  Result.success("查询成功！",orderInfo);
    }

    @PostMapping("/deleteSysRole/{id}")
    @ApiOperation(value = "删除角色")
    public Result<Object> deleteSysRole(HttpServletRequest request,@RequestHeader("token") String token,
                                 @PathVariable("id") Long id){
        try {
            if( this.sysRoleService.deleteById(id)){
                return  Result.success("删除成功！");
            } else {
                return Result.error("角色被引用，删除失败！");
            }

        } catch (Exception e) {
            log.error("角色管理: 删除出现异常",e);
            return Result.error("删除失败！");
        }
    }
}
